/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--
--	3. This notice must not be removed or altered.
*/
package com.dbtimes.nflsourceloader

import org.apache.spark.sql.SparkSession
import com.typesafe.config.{ConfigFactory, Config}
import com.dbtimes.dw.sourceloader.DataSourceLoader

object SrcLoaderNFL {

  def main(args: Array[String]) {

    val configFileName = if (args.length == 1) args(0) else "sourceLoadDefault.json"

    val appConfig = ConfigFactory.load(configFileName)

    val sparkSessionBuilder = SrcLoaderNFL.configureSparkSession(appConfig)
    val spark = sparkSessionBuilder.getOrCreate()

    DataSourceLoader.loadData(appConfig)

    spark.stop()
  }

  def configureSparkSession(appConfig: Config): SparkSession.Builder = {
    var sparkSessionBuilder = SparkSession.builder()
    sparkSessionBuilder = if (appConfig.hasPath("sparkParams.sessionAppName")) sparkSessionBuilder.appName(appConfig.getString("sparkParams.sessionAppName")) else sparkSessionBuilder
    sparkSessionBuilder = if (appConfig.hasPath("sparkParams.sessionConfigMaster")) sparkSessionBuilder.config("spark.master", appConfig.getString("sparkParams.sessionConfigMaster")) else sparkSessionBuilder
    //  .config("spark.driver.host", "localhost")  // to get rid of this error when running not connected to the internet. ERROR SparkContext: Error initializing SparkContext. java.lang.AssertionError: assertion failed: Expected hostname (not IP) but got 2600:1001:b028:7e92:b449:4c2d:7be2:d6e8
    sparkSessionBuilder = if (appConfig.hasPath("sparkParams.sessionConfigDriverHost")) sparkSessionBuilder.config("spark.driver.host", appConfig.getString("sparkParams.sessionConfigDriverHost")) else sparkSessionBuilder
    sparkSessionBuilder = if (appConfig.hasPath("sparkParams.sessionConfigLocalDir")) sparkSessionBuilder.config("spark.local.dir", appConfig.getString("sparkParams.sessionConfigLocalDir")) else sparkSessionBuilder

    sparkSessionBuilder
  }

}
