/*
--  Copyright (c) 2007 - 2021 by Victor Shulman.
--  Written by Victor Shulman.  Not derived from licensed software.
--
--  Permission is granted to anyone to use this software for any
--  purpose on any computer system, and to redistribute it in any way,
--  subject to the following restrictions:
--
--  1. The author is not responsible for the consequences of use of
--    this software, no matter how awful, even if they arise
--    from defects in it.
--
--  2. The origin of this software must not be misrepresented, either
--    by explicit claim or by omission.
--
--  3. This notice must not be removed or altered.
*/
package com.dbtimes.nfldw.datamartgames

import com.dbtimes.dw.common.LogFile.{logger => dwEtlLog}
import com.dbtimes.nfldw.DwNFL

import org.apache.spark.sql.DataFrame

/**
 *
 * @param factName
 */
class FactPlaysForMerge(factName: String) extends FactsNFL(factName) {

  override def loadFact(stgSrcTempView: String): Option[DataFrame] = {

    val dfFact = if (stgSrcTempView == "PlayByPlay") {
      val sqlStgSource =
        s"""|SELECT
            |  src.SeasonYear                                                                          AS SeasonYear,
            |  "$stgSrcTempView"                                                                       AS DataSourceMoniker,
            |  CAST( src.GameId AS STRING )                                                            AS GameId,
            |  CAST( src.GameDate AS DATE )                                                            AS PlayDate,
            |  CASE WHEN src.OffenseTeam IS NOT NULL AND NOT( src.PlayType <=> 'TIMEOUT' )
            |       THEN src.OffenseTeam
            |       ELSE ''
            |       END                                                                                AS OffenseTeamAbbrev          ,
            |  CASE WHEN src.DefenseTeam IS NOT NULL AND NOT( src.PlayType <=> 'TIMEOUT' )
            |        THEN src.DefenseTeam
            |       ELSE ''
            |       END                                                                                AS DefenseTeamAbbrev          ,
            |  IFNULL( src.PenaltyTeam, '' )                                                           AS PenaltyTeamAbbrev          ,
            |  CASE WHEN src.PlayType = 'TIMEOUT' AND INSTR( 'BY ', src.Description ) > 0
            |       THEN RTRIM( SUBSTRING( src.Description, INSTR( 'BY ', src.Description ) + 3, 3 ) )
            |       ELSE ''
            |       END                                                                                AS TimeoutTeamAbbrev          ,
            |  IFNULL( src.Formation        , '' )                                                     AS Formation    ,
            |  IFNULL( src.PlayType      , '' )                                                        AS PlayType    ,
            |  IFNULL( src.PassType    , '' )                                                          AS PassType    ,
            |  IFNULL( src.PenaltyType    , '' )                                                       AS PenaltyType  ,
            |  CAST( src.ToGo    AS DOUBLE )                                                           AS ToGo                  ,
            |  CAST( src.PenaltyYards      AS DOUBLE )                                                 AS PenaltyYards
            |FROM $stgSrcTempView AS src
            |   INNER JOIN $viewWithDatesToProcess AS dates   ON  src.SeasonYear = udfSeasonYear( dates.$effDateColumnNameInDatesToProcess )
            |
            | """.stripMargin

      dwEtlLog.info(s"-- Loading fact $factName from source $stgSrcTempView using sql:\n" + sqlStgSource)

      val dfStgSource = spark.sql(sqlStgSource)

      dfStgSource.dropDuplicates()
    }
    else {
      throw new RuntimeException(s"""Etl ERROR: Unknown stg source while loading fact $factName""")
    }

    if (DwNFL.getIsDebug) {
      dfFact.printSchema()
      dfFact.show(40)
    }

    Some(dfFact)
  }

}
