/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--
--	3. This notice must not be removed or altered.
*/
package com.dbtimes.nfldw.dimauth

import com.dbtimes.dw.etl.Dim
import org.apache.spark.sql.DataFrame

class DimGame(dimName: String) extends Dim(dimName) {

  override def loadDim(stgSrcTempView: String): Option[DataFrame] = {
    val dfStgSourceAsOption = if (stgSrcTempView == "PlayByPlay") {
      val sqlPlayByPlay =
        """
          | SELECT
          |  CAST( NULL AS INT ) AS GameKey,
          |  CAST( GameId AS STRING ),
          |  MAX( CONCAT( OffenseTeam,'/',DefenseTeam ) )
          | FROM PlayByPlay
          | GROUP BY GameId """.stripMargin

      Some(spark.sql(sqlPlayByPlay))
    }
    else if (stgSrcTempView == "Teams") {
      None // Do not need to process this source
    }
    else
      throw new RuntimeException("""Dim Etl ERROR: Unknown stg source while loading dimension """)

    dfStgSourceAsOption
  }

}
