/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--
--	3. This notice must not be removed or altered.
*/
package com.dbtimes.nfldw.dimauth

import com.dbtimes.dw.etl.Dim
import org.apache.spark.sql.DataFrame

class DimPlayDetails(dimName: String) extends Dim(dimName) {

  override def loadDim(stgSrcTempView: String): Option[DataFrame] = {
    val sqlStgSource = if (stgSrcTempView == "PlayByPlay") {
      """|SELECT DISTINCT
         |	 IFNULL( CASE IsRush								          WHEN '0' THEN 'N' WHEN '1' THEN 'Y' ELSE IsRush								           END , '' )	AS IsRush							  ,
         |	 IFNULL( CASE IsPass								          WHEN '0' THEN 'N' WHEN '1' THEN 'Y' ELSE IsPass								           END , '' )	AS IsPass							  ,
         |	 IFNULL( CASE IsIncomplete					          WHEN '0' THEN 'N' WHEN '1' THEN 'Y' ELSE IsIncomplete					           END , '' )	AS IsIncomplete					,
         |	 IFNULL( CASE IsTouchdown					            WHEN '0' THEN 'N' WHEN '1' THEN 'Y' ELSE IsTouchdown					           END , '' ) AS IsTouchdown					,
         |	 IFNULL( CASE IsSack								          WHEN '0' THEN 'N' WHEN '1' THEN 'Y' ELSE IsSack								           END , '' ) AS IsSack							  ,
         |	 IFNULL( CASE IsChallenge					            WHEN '0' THEN 'N' WHEN '1' THEN 'Y' ELSE IsChallenge					           END , '' ) AS IsChallenge					,
         |	 IFNULL( CASE IsChallengeReversed	            WHEN '0' THEN 'N' WHEN '1' THEN 'Y' ELSE IsChallengeReversed	           END , '' ) AS IsChallengeReversed	,
         |	 IFNULL( CASE IsMeasurement				            WHEN '0' THEN 'N' WHEN '1' THEN 'Y' ELSE IsMeasurement				           END , '' ) AS IsMeasurement				,
         |	 IFNULL( CASE IsInterception				          WHEN '0' THEN 'N' WHEN '1' THEN 'Y' ELSE IsInterception				           END , '' ) AS IsInterception				,
         |	 IFNULL( CASE IsFumble							          WHEN '0' THEN 'N' WHEN '1' THEN 'Y' ELSE IsFumble							           END , '' ) AS IsFumble						  ,
         |	 IFNULL( CASE IsPenalty						            WHEN '0' THEN 'N' WHEN '1' THEN 'Y' ELSE IsPenalty						           END , '' ) AS IsPenalty						,
         |	 IFNULL( CASE IsTwoPointConversion	          WHEN '0' THEN 'N' WHEN '1' THEN 'Y' ELSE IsTwoPointConversion	           END , '' )	AS IsTwoPointConversion	,
         |	 IFNULL( CASE IsTwoPointConversionSuccessful	WHEN '0' THEN 'N' WHEN '1' THEN 'Y' ELSE IsTwoPointConversionSuccessful	 END , '' )	AS IsTwoPointConversionSuccessful	,
         |	 IFNULL( CASE IsPenaltyAccepted					      WHEN '0' THEN 'N' WHEN '1' THEN 'Y' ELSE IsPenaltyAccepted					     END , '' )	AS IsPenaltyAccepted		,
         |	 IFNULL( CASE IsNoPlay					              WHEN '0' THEN 'N' WHEN '1' THEN 'Y' ELSE IsNoPlay					               END , '' ) AS IsNoPlay
         |FROM PlayByPlay """.stripMargin
    }
    else
      throw new RuntimeException("""Dim Etl ERROR: Unknown stg source while loading dimension """)

    val dfStgSource = spark.sql(sqlStgSource)

    Some(dfStgSource)
  }

}
