/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--
--	3. This notice must not be removed or altered.
*/
package com.dbtimes.dw.common

import java.text.SimpleDateFormat
import java.text.DateFormat
import java.util.Date

import com.typesafe.config.{Config, ConfigValue}
import org.apache.commons.io.FilenameUtils
import org.apache.spark.sql.{DataFrame, Row}
import org.apache.spark.sql.catalyst.parser.CatalystSqlParser
import org.apache.spark.sql.functions.udf
import org.apache.spark.sql.types.{StructField, StructType}

import scala.collection.JavaConverters._ // is needed for .asScala method when getting a list


object DataFrameHelper {

  val concatColumns = udf((r: Row) => {
    val s = r.mkString("~")
    s
  })

  /**
   * df source DataFrame
   */
  implicit class DataFrameImplicits(df: DataFrame) {

    /**
     * Set nullable property of column.
     *
     * @param colName  is the column name to change
     * @param nullable is the flag to set, such that the column is either nullable or not
     */
    def setNullableStateOfColumn(colName: String)(nullable: Boolean): DataFrame = {

      // get schema
      val schema = df.schema
      // modify [[StructField] with name `colName`
      val newSchema = StructType(schema.map {
        case StructField(c, t, _, m) if c.equals(colName) => StructField(c, t, nullable = nullable, m)
        case y: StructField => y
      })
      // apply new schema
      df.sqlContext.createDataFrame(df.rdd, newSchema)
    }

    def setNullableStateForAllColumns(nullable: Boolean): DataFrame = {
      // get schema
      val schema = df.schema
      // modify [[StructField] with name `cn`
      val newSchema = StructType(schema.map {
        case StructField(c, t, _, m) => StructField(c, t, nullable = nullable, m)
      })
      // apply new schema
      df.sqlContext.createDataFrame(df.rdd, newSchema)
    }

    def setNewSchema(newSchema: StructType): DataFrame = {
      // apply new schema
      df.sqlContext.createDataFrame(df.rdd, newSchema)
    }

  }

}
