/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--
--	3. This notice must not be removed or altered.
*/
package com.dbtimes.dw.sourceloader

import com.dbtimes.dw.common.LogFile
import com.typesafe.config.ConfigValue
import org.apache.spark.sql.SparkSession
import com.typesafe.config.{ConfigFactory, Config, ConfigObject, ConfigValueType}
import scala.collection.JavaConverters._ // is needed for .asScala method when getting a list

import LogFile.{logger => loaderLog} // rename to loaderLog

object DataSourceLoader {

  def loadData(appConfig: Config): Unit = {

    val spark = SparkSession.builder().getOrCreate() // this gets previously created session
    val appName = try {
      spark.sparkContext.appName
    }
    catch {
      case e : Exception => ""
    }

    // Create log file if logFileDir is defined
    if ( appConfig.hasPath("sourceLoad.logFileDir") ) {
      LogFile.init( appName, "SOURCE_LOADER", None, appConfig.getString("sourceLoad.logFileDir"))
    }

    loaderLog.info("---- Starting load for application " + appName )

    // loop trough all configurations and perform the loads
    val loadActionConfigs: List[ConfigValue] = appConfig.getList("sourceLoad.loadActions").asScala.toList
    loadActionConfigs.foreach(new LoadAction(_).validate)
    loadActionConfigs.foreach(processLoadAction)

    loaderLog.info("---- Completed load for application " + appName )
  }

  private def processLoadAction(loadActionConfig: ConfigValue): Unit = {
    val action = new LoadAction(loadActionConfig)
    loaderLog.info("---- Processing load action: " + action.getName)
    if (!action.getIsActive) {
      loaderLog.info("Skipping processing - the action is inactive")
    }
    else { // active action - process it
      ActionProcessorFactory.getLoadActionProcessor(action).process
    }
  }
}
