/*
--	Copyright (c) 2007 - 2021 by Victor Shulman.
--	Written by Victor Shulman.  Not derived from licensed software.
--
--	Permission is granted to anyone to use this software for any
--	purpose on any computer system, and to redistribute it in any way,
--	subject to the following restrictions:
--
--	1. The author is not responsible for the consequences of use of
--		this software, no matter how awful, even if they arise
--		from defects in it.
--
--	2. The origin of this software must not be misrepresented, either
--		by explicit claim or by omission.
--
--	3. This notice must not be removed or altered.
*/
package com.dbtimes.dw.sourceloader

import com.dbtimes.dw.common._
import org.apache.spark.sql.SparkSession
import LogFile.{logger => loaderLog}

private[sourceloader] class ProcessorOfCsvFileSource(private val action: LoadAction) extends ActionProcessor
  with ProcessFileSourceAny
  with SourceDataMerger {

  private[sourceloader] def process(): Unit = {

    // get a list of all files that match the patter sorted by effective date and process them
    val fileList = getFilesToProcessSortedByEffectiveDate(action)
    fileList.foreach(processFile)
  }

  private[sourceloader] def processFile(sourceFilePath: String): Boolean = {
    val spark = SparkSession.builder().getOrCreate() // this gets previously created session

    loaderLog.info("** Processing file " + sourceFilePath)
    val dfNewData = spark
      .read.format("com.databricks.spark.csv")
      .option("mode", "PERMISSIVE") // need "PERMISSIVE" because some columns do not have a name in the header and FAILFAST option fails to load
      .option("header", action.getHasHeader)
      .schema(action.getSchema)
      .load(sourceFilePath)

    val effectiveDate = action.getEffectiveDate(dfNewData, sourceFilePath)

    mergeSourceData(action, dfNewData, effectiveDate, loaderLog)

    moveFileToProcessedFolder(action, sourceFilePath)


    //    val processedFilePath =
    //      + new SimpleDateFormat("yyyyMMdd_hhmmss").format(Calendar.getInstance().getTime())
    //    + java.util.UUID.randomUUID.toString
    loaderLog.info("** Completed processing file " + sourceFilePath)
    true

  }

}
